#!/usr/bin/perl
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted
# material is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Milan Zazrivec

# general approach of the test:
# 1) Find out where's the install logfile to be scanned
# 2) Read the number of installed packages from the first line.
#    The first line looks as follows: "Installing 286 packages"
#    or locale's equivalent depending on the locale being used during
#    installation. We actually care only about the number of pkgs. ($pkg_cnt).
# 3) Read lines ($line) in loop and have previous read line saved ($prev_line),
#    i.e. at the beginning of every iteration we save previous line and read 
#    a new one. In this step, we try to read all "Installing pkg-ver-rel-arch."
#    lines, with all lines containing warnings and errors occured during
#    package installation. So inside the loop:
#    a) If $prev_line and $line match $install_line pattern (i.e. mentioned 
#       "Install ..." line or its locale's equivalent), we know we have found
#       two good looking lines and we'll just skip the rest of the loop.
#    b) If $prev_line matches $install_line pattern and $line is an empty
#       line, we know we have reached the end of the interesting section
#       and we'll just skip the rest of the file (it's absolutelly insignificant
#       for the purpose of the test).
#    c) If $prev_line matches $install_line pattern and $line matches 
#       $non_error_logs pattern (messages usually produced during upgrade,
#       not to be considered as error messages), we just read all
#       $non_error_logs - like lines and then roll back to the beginning
#       of iteration.
#       Interesting part: If we choose non-english language for install, all
#       "Installing pkg-ver-rel-arch." lines are translated into that locale.
#       All infos, warnings and errors produced by rpm during install are
#       always english.
#    d) If $prev_line matches $install_line pattern and $line does not
#       look like $non_error_logs line, we know we have found error lines
#       in install log and we'll print that stuff to stdout.


use strict;
use warnings;

print "Running install.log test\n\n".
      "Purpose of this test is to scan installation logfile created by\n".
      "anaconda (usually located in /root/install.log) and report\n".
      "errors occured during installation (reporting PASS if there were\n".
      "no errors found, FAIL otherwise)\n\n";

my $install_log_new = "/root/install.log";
my $install_log_old = "/tmp/install.log";
my $install_log;

my $line;
my $prev_line;
my $errors = 0;

# $install_line is common "Installing N-V-R.A" pattern
my $install_line = qr/^.+\ (.+)\-(.+)\-(.+)\.([^.]+)\.{0,1}$/x;
# $non_error_logs is pattern matching all post $inst_line messages
# not to be considered as error log
my $non_error_logs = qr/^Upgrade\ Dependency:|^Upgrade:|^install\-info:|
                     .*created\ as.*rpmnew|.*saved\ as.*rpmsave$|
                     .*saved\ as.*rpmsave$/x;
# variables for storing N-V-R.A of last succesfully matched $install_line
my $name;
my $version;
my $release;
my $arch;

# files
local *LOG_FILE;

# we can call the test with an optional install_log_file argument
if (defined($ARGV[0])) # did we provide any parameter to the file?
{
    open LOG_FILE, $ARGV[0] or die "$ARGV[0]: $!\n";
    $install_log = $ARGV[0];
}
elsif (open LOG_FILE, $install_log_new) # searching for install.log files
{
    $install_log = $install_log_new;
}
elsif (open LOG_FILE, $install_log_old)
{
    $install_log = $install_log_old;
}
else
{
    die "Did not find any install.log file to scan\n";
}

print "Scanning $install_log:\n";

die "$!\n" if not defined ($line = <LOG_FILE>);
chomp $line;
while ($line !~ $install_line)
{
    $line = <LOG_FILE>;
    chomp $line;
}

while ($prev_line = $line, $line = <LOG_FILE>)
{
    chomp $line;
    if ($prev_line =~ $install_line)
    {
        $name = $1;
        $version = $2;
        $release = $3;
        $arch = $4;
        if ($line =~ $install_line) # "Installing...\nInstalling...\n"
        {
            next; # skip the rest of the loop
        }
        if ($line =~ /^$/) # last "Installing..." record
        {
            last; # we don't care about the rest of the file
        }
        if ($line =~ $non_error_logs) # common upgrade logs not to be
        {                             # considered as errors
            while ($line =~ $non_error_logs and 
                   $line !~ $install_line and $line !~ /^$/)
            {
                die "$!" if (not $line = <LOG_FILE>);
                chomp $line;
            }
            redo; # roll back
        }
    }
    if ($prev_line =~ $install_line and not $line =~ $non_error_logs)
    {   # we found error logs for $prev_line package
        ++$errors;
        print "\nError(s)/warning(s) occured while installing ".
              "$name-$version-$release\.$arch\n$line\n";
        while (($line = <LOG_FILE>), chomp($line), ($line !~ /^$/) and
               ($line !~ $install_line ) and ($line !~ $non_error_logs))
        {   # print all errors until we find another "Installing..." line
            print "$line\n"; 
        }
    }
}

close LOG_FILE;

print "\nTest complete: result=PASS\n" if not $errors;
print "\nTest complete: result=FAIL\n" if $errors;

exit $errors;
